#version 430 core
out vec4 FragColor;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 texCoord;
	vec3 T;
	vec3 B;
	vec3 N;
} vert;

layout (location = 1) uniform vec4 color;
uniform sampler2D texture_diffuse1;
uniform sampler2D texture_normal1;
layout(location = 5) uniform samplerCube skybox;
uniform bool environment_mapping;
uniform bool normal_mapping;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform vec3 diffuseColor;
uniform vec3 camera_world;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirLight;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main()
{
	
	vec3 n = vert.N;
	if (normal_mapping)
	{
		n = texture(texture_normal1, vert.texCoord).rgb;
		n = normalize(n * 2.0 - 1.0);  // this normal is in tangent space
		mat3 TBN = mat3(normalize(vert.T), normalize(vert.B), normalize(vert.N));
		n = normalize(TBN * n);
	}

	vec3 v = normalize(camera_world - vert.position_world);
	vec3 c = texture(texture_diffuse1, vert.texCoord).rgb * diffuseColor;

	// environment mapping
	if (environment_mapping)
	{
		vec3 i = normalize(vert.position_world - camera_world);
		vec3 r = reflect(i, normalize(vert.normal_world));
		FragColor = vec4(texture(skybox, r).rgb, 1.0) * 0.2;
	}

	FragColor += vec4(c * materialCoefficients.x, 1); // ambient

	// add directional light contribution
	FragColor.rgb += phong(n, -dirLight.direction, v, dirLight.color * c, materialCoefficients.y, dirLight.color, materialCoefficients.z, specularAlpha, false, vec3(0));
}